<?php
/*
 * Plugin Name: Favatar Plus
 * Plugin URI: http://schepers.cc/plugins/favatar
 * Description: Add distributed, commenter-controlled favatars to your comments, with a fallback to Gravatars or identicons.  Commenters simply place an image named 'favatar.png' on their site in the directory they indicate in the comment URL field, and it will be fetched automatically.
 * Version: 1.0
 * License: GPL2
 * Author: Doug Schepers
 * Author URI: http://schepers.cc
 * Min WP Version: 2.0.4
 *      
 * Code based on "Easy Gravatars" Wordpress plugin by Dougal Campbell http://dougal.gunters.org/
 */

// Register our activation hook, so we can set our default options:
register_activation_hook(_FILE_,'favatar_activate');

function favatar_activate() {
	// ...but only if we don't already have options set...
	$favatar_opt_size = get_option('favatar_size');
	if ( empty($favatar_opt_size) ) {
		$defaults = favatar_defaults();
		foreach ($defaults as $key => $val) {
			update_option($key, $val);
		}
	}
}

// Let WP know that we have an addition to the admin menus:
add_action('admin_menu', 'favatar_opt_menu');

function favatar_opt_menu() {
	add_options_page('Favatar Plus', 'Favatar Plus', 'manage_options', 'testoptions', 'favatar_options_page');
}

// Add in our custom CSS
add_action('admin_head', 'favatar_admin_css');

function favatar_admin_css() {
        $plugindir = get_option('siteurl') . '/wp-content/plugins/' . dirname(plugin_basename(__FILE__));
        $favatar_css = $plugindir . '/favatar_plus.css';
?>
<link rel="stylesheet" type="text/css" href="<?php echo $favatar_css; ?>" />
<?php

}

// Set the filter requested by the user:
add_action('init', 'favatar_set_filter');

function favatar_set_filter() {
	$opts = favatar_get_options();
	$filter = $opts['favatar_api_hook'];
	add_filter($filter, 'getAvatar');
}

// Returns a hash of the default option values
function favatar_defaults() {
	return array(
		'favatar_size' => 80,
		'favatar_rating' => 'G',
		'favatar_defaulturl' => '',
		'favatar_style_span' => 'float:right; margin-left:10px; display:block',
		'favatar_style_img' => '',
                'favatar_api_hook' => 'get_comment_author_link',
	);
}

// return array of plugin options, using defaults if necessary:
function favatar_get_options() {
	$favatar_size = get_option('favatar_size');
	$favatar_rating = get_option('favatar_rating');
	$favatar_defaulturl = get_option('favatar_defaulturl');
	$favatar_style_span = get_option('favatar_style_span');
	$favatar_style_img = get_option('favatar_style_img');
	$favatar_api_hook = get_option('favatar_api_hook');
	
	$defaults = favatar_defaults();
	// Extra paranoia:
	if(empty($favatar_size))
		$favatar_size = $defaults['favatar_size'];
	if(empty($favatar_rating))
		$favatar_rating = $defaults['favatar_rating'];
	if(empty($favatar_defaulturl))
		$favatar_defaulturl = $defaults['favatar_defaulturl'];
	if(empty($favatar_style_span))
		$favatar_style_span = $defaults['favatar_style_span'];
	if(empty($favatar_style_img))
		$favatar_style_img = $defaults['favatar_style_img'];
        if(empty($favatar_api_hook))
                $favatar_api_hook = $defaults['favatar_api_hook'];
		
	return array(
		'favatar_size' => $favatar_size,
		'favatar_rating' => $favatar_rating,
		'favatar_defaulturl' => $favatar_defaulturl,
		'favatar_style_span' => $favatar_style_span,
		'favatar_style_img' => $favatar_style_img,
		'favatar_api_hook' => $favatar_api_hook,
	);
}

// Filter to add an avatar image, prepended:
function getAvatar($text) {
	global $comment;
	
	$commenturl = $comment->comment_author_url;
	$email = $comment->comment_author_email;
	if ( !empty( $commenturl ) || !empty( $email ) ) {
		$opts = favatar_get_options();

		$favatar_size = $opts['favatar_size'];
		$default = $opts['favatar_defaulturl'];
		$favatar_style_span = $opts['favatar_style_span'];
    $favatar_img_class = "";
    
	  // first see if favatar exists		
  	if ( !( empty($commenturl) || 'http://' == $commenturl ) ) {
  	  // check if the URL supplied includes the favatar reference
  	  if (strpos($commenturl, "favatar.png") !== false) {
    		$imglink = $commenturl;
        $favatar_img_class = "favatar";
      } else {
        // otherwise append the favatar string to URL
    	  $imglinktmp = $commenturl.'/favatar.png';
      	if( url_validate($imglinktmp) ) {
      		$imglink = $imglinktmp;
          $favatar_img_class = "favatar";
      	} else {
      	  $imglinktmp = $commenturl.'/.well-known/favatar.png';
        	if( url_validate($imglinktmp) ) {
        		$imglink = $imglinktmp;
            $favatar_img_class = "favatar";
        	}
      	}
      } 
	  }
		
    if ( !($imglink) && !empty( $comment->comment_author_email ) ) {
      $favatar_rating = $opts['favatar_rating'];
      if (function_exists('get_avatar')) {
        // WP 2.5+
        $imgtag = get_avatar($email, $favatar_size, $default);
      } else {
        $md5 = md5( strtolower( $email ) );
        $default = urlencode( $default );
        $imglinktmp = 'http://www.gravatar.com/avatar.php?gravatar_id=$md5&amp;size=$favatar_size&amp;rating=$favatar_rating&amp;default=$default';
        if( url_validate($imglinktmp) ) {
          $imglink = $imglinktmp;
        }
      }
    }
	  
  	if ( !($imglink) && !($imgtag) && !( empty($commenturl) || 'http://' == $commenturl ) ) {
  	  $imglinktmp =  $commenturl.'/favicon.ico';
    	if( url_validate($imglinktmp) ) {
    	  $favatar_size = "30";
    		$imglink = $imglinktmp;
    	}
	  }
		
  	if ( $imglink && !($imgtag) ) {
  		$imgtag = "<img src='$imglink' width='$favatar_size' height='$favatar_size' alt='' class='avatar avatar-{$favatar_size} {$favatar_img_class}' />";
      
	  }
		
	  
  	if ( $imgtag ) {
  		if (! ( empty($commenturl) || 'http://' == $commenturl ) ) {
  		  $linkurl = $commenturl;
        // // if they have a default vCard called 'contact.vcf' at their URL, link to that?
        // if ( !( empty($commenturl) || 'http://' == $commenturl ) ) {
        //   $linktmp = $commenturl.'/contact.vcf';
        //        if( url_validate($linktmp) ) {
        //          $linkurl = $linktmp;
        //        }
        //      }
  			$imgtag = "<a rel='external nofollow' href='$commenturl'>$imgtag</a>";
  		}

  		$text = "<span class='favatar-image' style='$favatar_style_span; width:{$favatar_size}px' >$imgtag</span>" . $text;
	  }
	}

	return $text;
}


function url_validate( $link ) {
		
	$url_parts = @parse_url( $link );

	if ( empty( $url_parts["host"] ) )
		return false;

	if ( !empty( $url_parts["path"] ) ) {
		$documentpath = $url_parts["path"];
	} else {
		$documentpath = "/";
	}

	if ( !empty( $url_parts["query"] ) )
		$documentpath .= "?" . $url_parts["query"];

	$host = $url_parts["host"];
	$port = $url_parts["port"];
	
	if ( empty($port) )
		$port = "80";

	$socket = @fsockopen( $host, $port, $errno, $errstr, 30 );
	
	if ( !$socket )
		return false;
		
	fwrite ($socket, "HEAD ".$documentpath." HTTP/1.0\r\nHost: $host\r\n\r\n");

	$http_response = fgets( $socket, 22 );

	$responses = "/(200 OK)|(30[0-9] Moved)/";
	if ( preg_match($responses, $http_response) ) {
		fclose($socket);
		return true;
	} else {
		return false;
	}

}


// Options update page:
function favatar_options_page() {
        if (! current_user_can('manage_options') ) {
          print "<div class='error'>Permission denied.</div>\n";
        }
        
	// defaults
	$favatar_ratings = array('G', 'PG', 'R', 'X');
	$maxsize = 80;
	$minsize = 1;
	$favatar_hooks = array(
	  'get_comment_author_link' => 'Comment Author Link',
	  'comment_text' => 'Comment Text',
        );
	
	// See if the user has posted us some information
	// If they did, this hidden field will be set to 'Y'
	if( $_POST[ 'favatar_submitted' ] == 'Y' ) {
		check_admin_referer('favatar-plus-update-options');
		// Read their posted value
		$favatar_rating = $_POST['favatar_rating'];
		$favatar_size = (int) $_POST['favatar_size'];
		$favatar_defaulturl = $_POST['favatar_defaulturl'];
		$favatar_style_span = $_POST['favatar_style_span'];
		$favatar_api_hook = $_POST['favatar_api_hook'];
		
		if ($favatar_size > $maxsize) {
			$favatar_size = $maxsize;
		}

		if ($favatar_size < $minsize) {
			$favatar_size = $minsize;
		}

		// Save the posted value in the database
		update_option( 'favatar_rating', $favatar_rating );
		update_option( 'favatar_size', $favatar_size );
		update_option( 'favatar_defaulturl', $favatar_defaulturl );
		update_option( 'favatar_style_span', $favatar_style_span );
		update_option( 'favatar_api_hook', $favatar_api_hook );

		// Put an options updated message on the screen

?>
<div class="updated"><p><strong><?php _e('Options saved.', 'favatar_trans_domain' ); ?></strong></p></div>
<?php
	} // endif
	
	$opts = favatar_get_options();

	// Get existing option values:
	$favatar_opt_rating = $opts['favatar_rating'];
	$favatar_opt_size = $opts['favatar_size'];
	$favatar_opt_defaulturl = $opts['favatar_defaulturl'];
	$favatar_opt_style_span = $opts['favatar_style_span'];
        $favatar_opt_api_hook = $opts['favatar_api_hook'];
	
// Main options form:
?>
<div class="wrap">
<h2><?php _e( 'Favatar Plus Plugin Options', 'favatar_trans_domain' ); ?></h2>

<form id="favatar_plus" name="favatar_opts" method="post" action="<?php echo str_replace( '%7E', '~', $_SERVER['REQUEST_URI']); ?>">
<p class="submit">
<input type="submit" name="Submit" value="<?php _e('Update Options', 'favatar_trans_domain' ) ?>" />
</p>

<?php wp_nonce_field('favatar-plus-update-options') ?>
<input type="hidden" name="favatar_submitted" value="Y">

<fieldset id="general">
<legend><?php _e("General Options", 'favatar_trans_domain'); ?></legend>
<p>
<label for="favatar_size"><?php _e("Size, in pixels (maximum: 80):", 'favatar_trans_domain' ); ?></label>
<input type="text" name="favatar_size" id="favatar_size" value="<?php echo $favatar_opt_size; ?>" size="4">
</p>

<p>
<label for="favatar_rating"><?php _e("Allowed Rating:", 'favatar_trans_domain' ); ?></label>
<select name="favatar_rating" id="favatar_rating">
<?php
	foreach ($favatar_ratings as $rating) {
		$selected = ($rating == $favatar_opt_rating) ? 'selected="selected"' : '';
		echo "\t<option value='$rating' $selected>$rating</option>\n";
	}
?>
</select>
</p>

<p>
<div style="float:right;margin-left:10px;width=<?php echo $favatar_opt_size ?>"><img src="<?php echo $favatar_opt_defaulturl; ?>" width="<?php echo $favatar_opt_size; ?>" height="<?php echo $favatar_opt_size; ?>" /></div>
<label for="favatar_defaulturl"><?php _e("Default image url:", 'favatar_trans_domain' ); ?></label>
<input type="text" name="favatar_defaulturl" id="favatar_defaulturl" value="<?php echo $favatar_opt_defaulturl; ?>" size="40">
</p>
</fieldset>

<fieldset id="advanced">
<legend><?php _e('Advanced Options', 'favatar_trans_domain'); ?></legend>
<p>
<label for="favatar_style_span"><?php _e("Span style:", 'favatar_trans_domain' ); ?></label>
<input type="text" name="favatar_style_span" id="favatar_style_span" value="<?php echo $favatar_opt_style_span; ?>" size="40">
</p>

<p>
<label for="favatar_api_hook"><?php _e("API Hook:", 'favatar_trans_domain' ); ?></label>
<select name="favatar_api_hook" id="favatar_api_hook">
<?php
	foreach ($favatar_hooks as $hook => $label) {
		$selected = ($hook == $favatar_opt_api_hook) ? 'selected="selected"' : '';
		echo "\t<option value='$hook' $selected>$label</option>\n";
	}
?>
</select>
</p>
</fieldset>

<p class="submit">
<input type="submit" name="Submit" value="<?php _e('Update Options', 'favatar_trans_domain' ) ?>" />
</p>

</form>
</div>

<?php
 
}

?>
